///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("CMP..."); fflush(stdout);

    /* Verify transitivity, f <= g and g <= h implies f <= h */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(h, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        if (fmpq_poly_cmp(f, g) > 0)
            fmpq_poly_swap(f, g);

        result = fmpq_poly_cmp(g, h) > 0 || fmpq_poly_cmp(f, h) <= 0;
        if (!result)
        {
            char * strf, * strg, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(g);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            printf("  h = %s\n", strh);
            free(strf);
            free(strg);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

