///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("CONTENT..."); fflush(stdout);
    
    /* c(a f) == a c(f) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f;
        fmpz_t anum, aden;
        mpq_t a;
        mpq_t x, y;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        anum = fmpz_init(bits / FLINT_BITS + 2);
        aden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(a);
        mpq_init(x);
        mpq_init(y);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpz_randtest(&anum, R, rand() % bits);
        fmpz_randtest_not_zero(&aden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(a), anum);
        fmpz_to_mpz(mpq_denref(a), aden);
        mpq_canonicalize(a);
        
        fmpq_poly_content(y, f);
        mpq_mul(y, a, y);
        mpz_abs(mpq_numref(y), mpq_numref(y));
        fmpq_poly_scalar_mul_mpq(f, f, a);
        fmpq_poly_content(x, f);
        
        result = mpq_equal(x, y);
        if (!result)
        {
            char * str1, * str2;
            str1 = mpq_get_str(NULL, 10, x);
            str2 = mpq_get_str(NULL, 10, y);
            printf("FAIL\n");
            printf("  x = %s\n", str1);
            printf("  y = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpz_clear(anum);
        fmpz_clear(aden);
        mpq_clear(a);
        mpq_clear(x);
        mpq_clear(y);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

