///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("EVALUATE_MPZ..."); fflush(stdout);
    
    /* Aliasing (f + g)(a) == f(a) + g(a) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t aa;
        mpz_t a;
        mpq_t x, y;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        aa = fmpz_init(bits / FLINT_BITS + 2);
        mpz_init(a);
        mpq_init(x);
        mpq_init(y);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpz_randtest(&aa, R, rand() % bits);
        fmpz_to_mpz(a, aa);
        
        fmpq_poly_evaluate_mpz(x, f, a);
        fmpq_poly_evaluate_mpz(y, g, a);
        mpq_add(y, x, y);
        fmpq_poly_add(f, f, g);
        fmpq_poly_evaluate_mpz(x, f, a);
        
        result = mpq_equal(x, y);
        if (!result)
        {
            char * str1, * str2;
            str1 = mpq_get_str(NULL, 10, x);
            str2 = mpq_get_str(NULL, 10, y);
            printf("FAIL\n");
            printf("  x = %s\n", str1);
            printf("  y = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(aa);
        mpz_clear(a);
        mpq_clear(x);
        mpq_clear(y);
    }
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

