///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("GETSLICE..."); fflush(stdout);
    
    /* Aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong i, j;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        i = rand() % (3 * fmpq_poly_length(f) / 2 + 1);
        j = rand() % (3 * fmpq_poly_length(f) / 2 + 1);
        
        fmpq_poly_getslice(g, f, i, j);
        fmpq_poly_getslice(f, f, i, j);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  g = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* slice(f + g) == slice(f) + slice(g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, lhs, rhs;
        ulong i, j;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        i = rand() % (3 * len / 2 + 1);
        j = rand() % (3 * len / 2 + 1);
        
        fmpq_poly_getslice(lhs, f, i, j);
        fmpq_poly_getslice(rhs, g, i, j);
        fmpq_poly_add(rhs, lhs, rhs);
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_getslice(lhs, lhs, i, j);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

