///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("INV..."); fflush(stdout);
    
    /* Check aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t num, den;
        mpq_t c;
        ulong len = 1, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        num = fmpz_init(bits / FLINT_BITS + 2);
        den = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(c);
        
        fmpz_randtest_not_zero(&num, R, rand() % bits);
        fmpz_randtest_not_zero(&den, R, rand() % bits);
        if (rand() % 2)
            fmpz_neg(num, num);
        fmpz_to_mpz(mpq_numref(c), num);
        fmpz_to_mpz(mpq_denref(c), den);
        mpq_canonicalize(c);
        fmpq_poly_set_coeff_mpq(f, 0, c);
        
        fmpq_poly_inv(g, f);
        fmpq_poly_inv(f, f);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(num);
        fmpz_clear(den);
        mpq_clear(c);
    }

    /* f * 1/f == 1 */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_t num, den;
        fmpq_poly_t f, g;
        mpq_t c;
        ulong len = 1, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        num = fmpz_init(bits / FLINT_BITS + 2);
        den = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(c);
        
        fmpz_randtest_not_zero(&num, R, rand() % bits);
        fmpz_randtest_not_zero(&den, R, rand() % bits);
        if (rand() % 2)
            fmpz_neg(num, num);
        fmpz_to_mpz(mpq_numref(c), num);
        fmpz_to_mpz(mpq_denref(c), den);
        mpq_canonicalize(c);
        fmpq_poly_set_coeff_mpq(f, 0, c);
        
        fmpq_poly_inv(g, f);
        fmpq_poly_mul(f, f, g);
        
        result = fmpq_poly_is_one(f);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(num);
        fmpz_clear(den); 
        mpq_clear(c);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

