///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 500;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("INV_SERIES..."); fflush(stdout);
    
    /* Aliasing f and g in g = f^{-1} mod t^n */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 50, n = len;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);

        fmpq_poly_randtest_not_zero(f, R, rand() % len, rand() % bits);
        if (fmpz_is_zero(fmpz_poly_get_coeff_ptr(fmpq_poly_numref(f), 0)))
            fmpq_poly_set_coeff_si(f, 0, -123);
        
        fmpq_poly_inv_series(g, f, n);
        fmpq_poly_inv_series(f, f, n);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  g = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* Check t * t^{-1} = 1 mod (t^n) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, t, u;
        ulong len = 100, bits = 50, n = len;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(t);
        fmpq_poly_init(u);
        
        fmpq_poly_randtest_not_zero(f, R, rand() % len, rand() % bits);
        if (fmpz_is_zero(fmpz_poly_get_coeff_ptr(fmpq_poly_numref(f), 0)))
            fmpq_poly_set_coeff_si(f, 0, -123);
        
        fmpq_poly_inv_series(g, f, n);
        fmpq_poly_mul(t, f, g);
        fmpq_poly_truncate(t, t, n);
        fmpq_poly_set_si(u, 1);

        result = fmpq_poly_equal(t, u);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  g = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(t);
        fmpq_poly_clear(u);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

