///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("ONE..."); fflush(stdout);
    
    /* Use is_one */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_one(f);
        
        result = fmpq_poly_is_one(f);
        if (!result)
        {
            char * strf;
            strf = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            free(strf);
            abort();
        }
        
        fmpq_poly_clear(f);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

