///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("POWER..."); fflush(stdout);
    
    /* Aliasing f and g in g = f ^ e */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100, ebits = 10;
        ulong e;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        e = rand() % ebits;
        
        fmpq_poly_power(g, f, e);
        fmpq_poly_power(f, f, e);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }
    
    /* Compute naively */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100, ebits = 10;
        ulong e;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        e = rand() % ebits;
        
        if (e == 0)
        {
            fmpq_poly_one(g);
        }
        else
        {
            fmpq_poly_set(g, f);
            ulong j;
            for (j = 1; j < e; j++)
                fmpq_poly_mul(g, g, f);
        }
        
        fmpq_poly_power(f, f, e);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

