///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("RESCALE..."); fflush(stdout);
    
    /* Check aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t anum, aden;
        mpq_t a;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        anum = fmpz_init(bits / FLINT_BITS + 2);
        aden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(a);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpz_randtest(&anum, R, rand() % bits);
        fmpz_randtest_not_zero(&aden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(a), anum);
        fmpz_to_mpz(mpq_denref(a), aden);
        mpq_canonicalize(a);
        
        fmpq_poly_rescale(g, f, a);
        fmpq_poly_rescale(f, f, a);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(anum);
        fmpz_clear(aden);
        mpq_clear(a);
    }

    /* Check that rescaling by a and then by 1/a is the identity */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t anum, aden;
        mpq_t a;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        anum = fmpz_init(bits / FLINT_BITS + 2);
        aden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(a);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpz_randtest_not_zero(&anum, R, rand() % bits);
        fmpz_randtest_not_zero(&aden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(a), anum);
        fmpz_to_mpz(mpq_denref(a), aden);
        mpq_canonicalize(a);
        
        fmpq_poly_rescale(g, f, a);
        mpq_inv(a, a);
        fmpq_poly_rescale(g, g, a);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(anum);
        fmpz_clear(aden);
        mpq_clear(a);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

