///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("RESULTANT..."); fflush(stdout);
    
    /* Check res(f, g) == (-1)^(deg f deg g) res(Q, P) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        mpq_t x, y;
        ulong len = 70, bits = 70;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        mpq_init(x);
        mpq_init(y);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_resultant(x, f, g);
        fmpq_poly_resultant(y, g, f);
        if ((fmpq_poly_degree(f) * fmpq_poly_degree(g)) % 2)
            mpz_neg(mpq_numref(y), mpq_numref(y));
        
        result = mpq_equal(x, y);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        mpq_clear(x);
        mpq_clear(y);
    }
    
    /* Check res(f h, g) == res(f, g) res(h, g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        mpq_t x, y, z;
        ulong len = 70, bits = 70;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        mpq_init(x);
        mpq_init(y);
        mpq_init(z);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(h, R, rand() % len, rand() % bits);
        
        fmpq_poly_resultant(y, f, g);
        fmpq_poly_resultant(z, h, g);
        mpq_mul(y, y, z);
        fmpq_poly_mul(f, f, h);
        fmpq_poly_resultant(x, f, g);
        
        result = mpq_equal(x, y);
        if (!result)
        {
            char * strf, * strg, * strh;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            printf("  h = %s\n", strh);
            free(strf);
            free(strg);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
        mpq_clear(x);
        mpq_clear(y);
        mpq_clear(z);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

