///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("set/ equal..."); fflush(stdout);

    /* Equal polynomials */    
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_set(g, f);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }
    
    /* Distinct polynomials */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t x, y;
        ulong len = 100, bits = 100;
        ulong i = rand() % len;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_set(g, f);
        
        x = fmpz_init(g->num->limbs);
        y = fmpz_init(bits / FLINT_BITS + 2);
        
        fmpz_poly_get_coeff_fmpz(x, g->num, i);
        do
            fmpz_randtest(&y, R, rand() % bits);
        while (fmpz_equal(x, y));
        fmpz_poly_set_coeff_fmpz(g->num, i, y);
        
        fmpq_poly_canonicalize(g, NULL);
        
        result = !fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(x);
        fmpz_clear(y);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

