///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("SWAP..."); fflush(stdout);
    
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f1, f2, g1, g2;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f1);
        fmpq_poly_init(f2);
        fmpq_poly_init(g1);
        fmpq_poly_init(g2);
        
        fmpq_poly_randtest(f1, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g1, R, rand() % len, rand() % bits);
        fmpq_poly_set(f2, f1);
        fmpq_poly_set(g2, g1);
        
        fmpq_poly_swap(f2, g2);
        
        result = fmpq_poly_equal(f1, g2) && fmpq_poly_equal(g1, f2);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f1);
            strg = fmpq_poly_to_string(g1);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f1);
        fmpq_poly_clear(f2);
        fmpq_poly_clear(g1);
        fmpq_poly_clear(g2);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

